# Title: Signal tower
# Version: V1.4
# Creation date: 28.04.2023 11:27
# Created: Trawicki Wojciech
# License Restrictions: Absence

# --------------------------------------------------------------------------

import time
Old_pos = d.getPosition(CoordMode.Machine)
Last_time = time.time()


# Settings -----------------------------------------------------------------

# Module type and number:
Modul_Type = ModuleType.IP									# For CSMIO/IP controller use "ModuleType.IP", for CSMIO-IO module use "ModuleType.IO"
Module_Number	= 0														# For CSMIO/IP controller use "0", for CSMIO-IO module use its number (0-15)


# Digital outputs:
Green_lamp_Out 			= 0											# Digital output number for the green lamp
Yellow_lamp_Out 			= 1											# Digital output number for the yellow lamp
Red_lamp_Out 				= 2											# Digital output number for the red lamp

Buzzer_Present					= True									# When the signal tower is equipped with a buzzer set to True if not set to False
Buzzer_Out 						= 3 											# Digital output number for the buzzer (Set to 0 if not used)

# Digital inputs:
External_buzzer_trigger_in = 1								# Digital input number triggering the buzzer (Set to 0 if not used)


# Interval settings:
Fast_flash_interval			= 250										# Fast flash interval for lamps, minimum value is 250ms.			
Slow_flash_interval		= 1000									# Slow flash interval for lamps, minimum value is 1000ms.		
Buzer_interval					= 1000									# Buzzer interval, minimum value is 250ms.		


# Functions - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

# Function checking whether the CSMIO/IP module or the CSMIO-IP module is available
def isModuleActive( moduleType, modIndex ):
	try:
		moduleList = d. getModuleList()
	except:
		return False
	if ( len(moduleList) == 0 ):
		return False
	for module in moduleList:
		if ( module.type == moduleType.name ) and ( int(module.id) == modIndex ):
			return True
	return False


# Function returns a name of the macro
def getMacroName():
	info = str(d.getCurrentScriptInfo( ))
	name = info.split("/")
	if ( len(name) == 1 ):
		name= info.split("\\")
	name = name[len(name) - 1]
	name = name.rstrip("}")
	name = name.rstrip("'")
	return name


# Function displays error
def printError(string):
	print("")
	print("-------------------------------------------------------------")
	print("Macro : " + getMacroName() + "	- Error")
	print("  >  " + string +" !")
	print("  >  Macro Stop")
	print("-------------------------------------------------------------")
	msg.err(string + " !")
	sys.exit(0)


# Function detects axis movement
def axesInMotion():
	Tolerance	 = 0.0002 # mm
	global	Old_pos

	if ( d.getMachineParam( 9000 ) == 20 ):
		Tolerance	 = Tolerance / 25.4
	
	Actual_pos = d.getPosition(CoordMode.Machine)
	for axis in range( 0,len(Actual_pos )):
		if (( Old_pos[axis] - Actual_pos[axis] < -Tolerance ) or ( Old_pos[axis] - Actual_pos[axis] > Tolerance )) :
			Old_pos = Actual_pos
			return True


# Function detects the tool change process
def toolChangeInProgres():
	if (str(d.getRunningScriptsInfo( )).count("m6.py") != 0): 
		return True


# Function detects buzzer activation
def externalBuzzer():
	if ( mod_X.getDigitalIO(IOPortDir.InputPort, External_buzzer_trigger_in) == DIOPinVal.PinSet ):
		return True


# Function turns on the digital output cyclically
def cyclic(out, interval):
	global Last_time
	interval = interval / 1000
	Current_time = time.time()
			
	if Current_time > (Last_time  + interval / 2):
		mod_X.setDigitalIO(out, DIOPinVal.PinReset)
	if Current_time > (Last_time  + interval):
		Last_time  = Current_time
		mod_X.setDigitalIO(out, DIOPinVal.PinSet)


# Function turn on digital output
def turnOn(out):
	try:
		mod_X.setDigitalIO(out, DIOPinVal.PinSet)
	except Exception as e:
		printError(str(e) + " (pin " + str(out) + ")")


# Function turn off digital output
def turnOff(out):
	try:
		mod_X.setDigitalIO(out, DIOPinVal.PinReset)
	except Exception as e:
		printError(str(e) + " (pin " + str(out) + ")")
	
# Macro START - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
print("")
print("-------------------------------------------------------------")
print("Macro : " + getMacroName() + "	- Start")
print("-------------------------------------------------------------")

# I check if CSMIO/IP module or CSMIO-IO module is active ---------------------------------------------

if isModuleActive( Modul_Type, Module_Number ): 
	print("  >  " + str(Modul_Type) + " number " + str(Module_Number) + "	= ACTIVE")
	mod_X = d.getModule( Modul_Type, Module_Number )
else:
	printError(str(Modul_Type) + " number " + str(Module_Number) + "	= INACTIVE")


# Check parameters ---------------	---------------------------------------------------------------------

if (Fast_flash_interval < 250): printError("Value 'Fast_flash_interval' must be >= 250")
if (Slow_flash_interval < 1000): printError("Value 'Slow_flash_interval' must be >= 1000")
if (Buzer_interval < 250): printError("Value 'Buzer_interval' must be >= 250")


# Displays parameters ---------------------------------------------------------------------------------

print("  >  Intervals :")
print("      - Fast_flash_interval	= " + str( Fast_flash_interval))
print("      - Slow_flash_interval	= " + str(Slow_flash_interval))
print("      - Buzer_interval	= " + str(Buzer_interval))
print("  >  Signals : ")
print("      - Green_lamp_Out	= " + str(Green_lamp_Out))
print("      - Yellow_lamp_Out	= " + str(Yellow_lamp_Out))
print("      - Red_lamp_Out	= " + str(Red_lamp_Out))
print("      - Buzzer_Present	= " + str(Buzzer_Present))
print("      - Buzzer_Out	= " + str(Buzzer_Out))
print("      - External_buzzer_trigger_in	= " + str(External_buzzer_trigger_in))


# My code ---------------------------------------------------------------------------------------------

while True :

	Motion = axesInMotion()
	
	# Buzer and Green lamp, Yellow lamp, Red lamp
	if ( externalBuzzer() == True ) and ( Buzzer_Present == True ):
		turnOn(Green_lamp_Out)
		turnOn(Yellow_lamp_Out)
		turnOn(Red_lamp_Out)	
		cyclic(Buzzer_Out, Buzer_interval)		

	else:
		# Buzer
		if (Buzzer_Present	== True ):
			turnOff(Buzzer_Out)

		# Green lamp
		if ( d.getState( ) == State.Trajectory ) or ( d.getState( ) == State.MDI ): 
			turnOn(Green_lamp_Out) 
		else:
			if ( Motion == True ) or ( d.getState( ) == State.MPG ):
				cyclic(Green_lamp_Out, Fast_flash_interval)
			else:
				turnOff(Green_lamp_Out)	

		# Yellow lamp
		if ( d.getTrajectoryPause( ) == True ):
			turnOn(Yellow_lamp_Out) 
		else:
			if ( toolChangeInProgres() == True ):
				cyclic(Yellow_lamp_Out, Slow_flash_interval)
			else:
				turnOff(Yellow_lamp_Out)

		# Red lamp
		if ( d.getState( ) == State.Idle and Motion != True):
			turnOn(Red_lamp_Out) 
		else: 
			if ( d.getState( ) == State.EStop ):
				cyclic(Red_lamp_Out, Slow_flash_interval)
			else:
				turnOff(Red_lamp_Out)	


	time.sleep(0.01)


